// out.cpp
// C++ conversion of topsites out script
// by Ben Congleton
// converted to C++ by Kevin Ferguson
// (c) 1999 Nethernet

#include <cstdlib>
#include <ctime>
#include <ex.h>
#include <fstream>
#include <iostream>
#include <map>
#include <string>
#include <unistd.h>
#include <vector>

//#define DEBUG

using namespace std;

string const bad_url( "http://www.mp3toplist.com" );
string const ac_dir( "/usr/local/apache/cgi-bin/topsites" );
string const page_dir( "../" );
string const defaults( "default.txt" );
string const locks( "/usr/local/apache/cgi-bin/topsites/locks" );
int const time_out( 10 );

class Exists : public unary_function<string const, bool> {
public:
  inline bool operator () ( string const str ) const {
    #if defined( DEBUG )
    cerr << "Testing existance of " << str << endl;
    #endif
    ifstream ifs( str.c_str() );
    return ifs.is_open();
  }
};

// a class encapsulating the HTTP environment - basically, a
// clean interface to all the information given by the HTTP
// environment variables.  methods should be self-explanatory.
// this is not general stuff; it's specific to this application
// this is freaky -- i'm commenting my code.  ah well first time
// for everything
class HTTPEnv {
public:
  HTTPEnv( void );

  string IP( void ) { return remote_ip; }
  string AccountO( void ) { return account_o; }
  string AccountF( void ) { return account_f; }
protected:
  string remote_ip;
  string account_f;
  string account_o;
};

HTTPEnv::HTTPEnv( void ) :
  remote_ip( getenv( "REMOTE_ADDR" ) ),
  account_f( getenv( "HTTP_COOKIE" ) ),
  account_o( getenv( "PATH_INFO" ) ) {
  unsigned i;

  if( account_f.find( "nn_topsites" ) >= 0 ) {
    account_f.erase( 0, account_f.find( "nn_topsites" ) );
    account_f.erase( 0, account_f.find( "=" ) + 1 );
    for( i = 0; i < account_f.size(); i++ )
      if( !isdigit( account_f[i] ) && !isalpha( account_f[i] ) )
	account_f.erase( i, 1 );
  } else account_f = "";
  for( i = 0; i < account_o.size(); i++ )
    if( !isdigit( account_o[i] ) && !isalpha( account_o[i] ) )
      account_o.erase( i, 1 );

  #if defined( DEBUG )
  cerr << "HTTPEnv constructed" << endl;
  cerr
    << " remote_ip=" << remote_ip << endl
    << " account_f=" << account_f << endl
    << " account_o=" << account_o << endl;
  #endif
}

class IPList {
public:
  IPList( string );

  bool Match( string );

protected:
  vector<string> ips;
};

bool IPList::Match( string s ) {
  unsigned i;
  for( i = 0; i < ips.size(); i++ )
    if( ips[i]==s ) return true;
  return false;
}

IPList::IPList( string loc ) :
  ips( 100 ) {
  #if defined( DEBUG )
  cerr << "Constructing IP list (" << loc << ")" << endl;
  #endif

  Exists exists;
  if( !exists( loc ) ) {
    ips.resize( 0 );
    return;
  }
  char tmp[16];
  ifstream ifs( loc.c_str() );
  int i( 0 ), size( 100 );

  ifs.getline( tmp, 16 );
  while( !ifs.eof() ) {
    #if defined( DEBUG )
    cerr << "Adding IP: " << tmp << endl;
    #endif
    ips[i++].assign( tmp );
    if( i >= size ) {
      size += 100;
      ips.resize( size );
    }
    ifs.getline( tmp, 16 );
  }
  ips.resize( i );
}

// not complete site info; just the info about which we care
class Site {
public:
  Site( string );

  void IncHitsOut( void ) { hitsout++; hitsout_total++; }
  void IncHitsIn( void ) { score++; hitsin_total++; }
  void Write( ofstream & );

  string GetURL( void ) const { return url; }
  void SetURL( string s ) { url = s; }
protected:
  int score;
  string title;
  string id;
  string url;
  string bannerurl;
  string description;
  string name;
  string password;
  string email;
  string awards;
  int hitsout;
  int hitsout_total;
  int hitsin_total;
  int days;
  string account_data;
};

Site::Site( string fname ) {
  #if defined( DEBUG )
  cerr << "Reading site info from " << fname << endl;
  #endif
  ifstream ifs( fname.c_str() );
  if( !ifs.is_open() ) throw Ex( Ex::FILE, string( "Couldn't open " ) + fname );
  char tmpline[512];
  ifs.getline( tmpline, 512 );
  account_data.assign( tmpline );
  string tmp;
  int i;

  #if defined( DEBUG )
  cerr << account_data << endl;
  #endif

  i = account_data.find( ";;;" );
  tmp.assign( account_data, 0, i );
  score = atoi( tmp.c_str() );
  #if defined( DEBUG )
  cerr << " Score: " << score << endl;
  #endif
  account_data.erase( 0, i + 3 );

  #if defined( DEBUG )
  cerr << account_data << endl;
  #endif

  i = account_data.find( ";;;" );
  title.assign( account_data, 0, i );
  account_data.erase( 0, i + 3 );

  #if defined( DEBUG )
  cerr << account_data << endl;
  #endif

  i = account_data.find( ";;;" );
  id.assign( account_data, 0, i );
  account_data.erase( 0, i + 3 );

  #if defined( DEBUG )
  cerr << account_data << endl;
  #endif
  
  i = account_data.find( ";;;" );
  url.assign( account_data, 0, i );
  #if defined( DEBUG )
  cerr << " URL: " << url << endl;
  #endif
  account_data.erase( 0, i + 3 );

  #if defined( DEBUG )
  cerr << account_data << endl;
  #endif

  i = account_data.find( ";;;" );
  bannerurl.assign( account_data, 0, i );
  account_data.erase( 0, i + 3 );

  i = account_data.find( ";;;" );
  description.assign( account_data, 0, i );
  account_data.erase( 0, i + 3 );

  i = account_data.find( ";;;" );
  name.assign( account_data, 0, i );
  account_data.erase( 0, i + 3 );

  i = account_data.find( ";;;" );
  password.assign( account_data, 0, i );
  account_data.erase( 0, i + 3 );

  i = account_data.find( ";;;" );
  email.assign( account_data, 0, i );
  account_data.erase( 0, i + 3 );

  i = account_data.find( ";;;" );
  awards.assign( account_data, 0, i );
  account_data.erase( 0, i + 3 );

  i = account_data.find( ";;;" );
  tmp.assign( account_data, 0, i );
  hitsout = atoi( tmp.c_str() );
  account_data.erase( 0, i + 3 );

  i = account_data.find( ";;;" );
  tmp.assign( account_data, 0, i );
  hitsout_total = atoi( tmp.c_str() );
  account_data.erase( 0, i + 3 );

  i = account_data.find( ";;;" );
  tmp.assign( account_data, 0, i );
  hitsin_total = atoi( tmp.c_str() );
  account_data.erase( 0, i + 3 );

  i = account_data.find( ";;;" );
  tmp.assign( account_data, 0, i );
  days = atoi( tmp.c_str() );
  account_data.erase( 0, i + 3 );
}

void Site::Write( ofstream &ofs ) {
  #if defined( DEBUG )
  cerr << "Writing site info for " << id << endl;
  #endif
  ofs
    << score << ";;;"
    << title << ";;;"
    << id << ";;;"
    << url << ";;;"
    << bannerurl << ";;;"
    << description << ";;;"
    << name << ";;;"
    << password << ";;;"
    << email << ";;;"
    << awards << ";;;"
    << hitsout << ";;;"
    << hitsout_total << ";;;"
    << hitsin_total << ";;;"
    << days << ";;;"
    << account_data << endl;
}

class FLock {
public:
  FLock( string fname, int num = 0 ) :
    fn( fname ),
    n( num ),
    locked( false ) {
    #if defined( DEBUG )
    cerr << "Locking " << fn << " (" << num << ")" << endl;
    #endif
    fn.erase( 0, fn.find_last_of( string( "/" ) ) + 1 );
    full = locks + "/" + fn + ".lock";
    if( !exists( full ) ) {
      ofstream ofs( full.c_str() );
      ofs << "locked : " << fn << ".lock - " << time( NULL );
      locked = true;
    } else {
      if( num > time_out ) return;
      sleep( 1 );
      FLock newlock( fname, num + 1 );
    }
  }

  ~FLock( void ) {
    if( locked ) {
      #if defined( DEBUG )
      cerr << "Unlocking " << fn << endl;
      #endif
      unlink( full.c_str() );
    }
  }
protected:
  string fn;
  string full;
  int n;
  Exists exists;
  bool locked;
};

class Out {
public:
  Out( void );
protected:
  HTTPEnv env;
  IPList ips, ipsa;
  bool l, ll;
  Site a, b;
};

Out::Out( void ) :
  ips( locks + "/" + env.AccountO() + ".ip.out" ),
  ipsa( locks + "/" + env.AccountF() + ".ip.from" ),
  l( ips.Match( env.IP() ) ),
  ll( ipsa.Match( env.IP() ) || !env.AccountF().size() ),
  a( ac_dir + "/" + env.AccountO() ),
  b( ac_dir + "/" + env.AccountF() ) {
  #ifdef DEBUG
  cerr << "Constructing Out" << endl;
  #endif
  if( !ll ) {
    ofstream
      ofs( string( locks + "/" + env.AccountF() + ".ip.from").c_str(), ios::app ),
      ofs2( string( locks + "/" + env.AccountO() + ".ip.out" ).c_str(), ios::app );
    ofs << env.IP() << endl;
    ofs2 << env.IP() << endl;
    
    a.IncHitsOut();

    FLock lock( ac_dir + "/" + env.AccountO() );     
    ofstream ofs3( string( ac_dir + "/" + env.AccountO() ).c_str() );
    a.Write( ofs3 );
  }
  if( !l ) {
    b.IncHitsIn();

    FLock loque( ac_dir + "/" + env.AccountF() );
    ofstream ofs4( string( ac_dir + "/" + env.AccountF() ).c_str() );
    b.Write( ofs4 );
  }
  if( int( a.GetURL().find( "http" ) ) < 0 ) {
    #if defined( DEBUG )
    cerr << a.GetURL() << ": invalid url, using bad_url" << endl;
    #endif
    a.SetURL( bad_url );
  }
  cout << "Location: " << a.GetURL() << endl << endl;
}

int main( void ) {
  try {
    Out out;
    return 0;
  } catch( Ex ex ) {
    #if defined( DEBUG )
    ex.Report();
    #endif
    cout << "Location: " << bad_url << endl << endl;
    return 1;
  }
}













